@@ -1,3 +1,4 @@
+{-# LANGUAGE PackageImports #-}
 
 module Apply(applyHints, applyHintFile, applyHintFiles) where
 
@@ -5,6 +6,7 @@ import Control.Applicative
 import Data.Monoid
 import HSE.All
 import Hint.All
+import GHC.Util
 import Idea
 import Data.Tuple.Extra
 import Data.Either
@@ -13,6 +15,8 @@ import Data.Maybe
 import Data.Ord
 import Config.Type
 import Config.Haskell
+import "ghc-lib-parser" HsSyn
+import qualified "ghc-lib-parser" SrcLoc as GHC
 import qualified Data.HashSet as Set
 import Prelude
 
@@ -36,28 +40,31 @@ applyHintFiles flags s files = do
 -- | Given a way of classifying results, and a 'Hint', apply to a set of modules generating a list of 'Idea's.
 --   The 'Idea' values will be ordered within a file.
 --
---   Given a set of modules, it may be faster pass each to 'applyHints' in a singleton list.
+--   Given a set of modules, it may be faster to pass each to 'applyHints' in a singleton list.
 --   When given multiple modules at once this function attempts to find hints between modules,
 --   which is slower and often pointless (by default HLint passes modules singularly, using
 --   @--cross@ to pass all modules together).
-applyHints {- PUBLIC -} :: [Classify] -> Hint -> [(Module SrcSpanInfo, [Comment])] -> [Idea]
+applyHints {- PUBLIC -} :: [Classify] -> Hint -> [ModuleEx] -> [Idea]
 applyHints cs = applyHintsReal $ map SettingClassify cs
 
-applyHintsReal :: [Setting] -> Hint -> [(Module_, [Comment])] -> [Idea]
+applyHintsReal :: [Setting] -> Hint -> [ModuleEx] -> [Idea]
 applyHintsReal settings hints_ ms = concat $
-    [ map (classify (cls ++ mapMaybe readPragma (universeBi m) ++ concatMap readComment cs) . removeRequiresExtensionNotes m) $
+    [ map (classify classifiers . removeRequiresExtensionNotes (hseModule m)) $
         order [] (hintModule hints settings nm m) `merge`
-        concat [order [fromNamed d] $ decHints d | d <- moduleDecls m] `merge`
-        concat [order [] $ hintComment hints settings c | c <- cs]
-    | (nm,(m,cs)) <- mns
+        concat [order [fromNamed d] $ decHints d | d <- moduleDecls (hseModule m)] `merge`
+        concat [order [declName $ GHC.unLoc d] $ decHints' d | d <- hsmodDecls $ GHC.unLoc $ ghcModule m]
+    | (nm, m) <- mns
+    , let classifiers = cls ++ mapMaybe readPragma (universeBi (hseModule m)) ++ concatMap readComment (ghcComments m)
+    , seq (length classifiers) True -- to force any errors from readPragma or readComment
     , let decHints = hintDecl hints settings nm m -- partially apply
-    , let order n = map (\i -> i{ideaModule= f $ moduleName m : ideaModule i, ideaDecl= f $ n ++ ideaDecl i}) . sortOn ideaSpan
+    , let decHints' = hintDecl' hints settings nm m -- partially apply
+    , let order n = map (\i -> i{ideaModule= f $ moduleName (hseModule m) : ideaModule i, ideaDecl = f $ n ++ ideaDecl i}) . sortOn ideaSpan
     , let merge = mergeBy (comparing ideaSpan)] ++
-    [map (classify cls) (hintModules hints settings $ map (second fst) mns)]
+    [map (classify cls) (hintModules hints settings mns)]
     where
         f = nubOrd . filter (/= "")
         cls = [x | SettingClassify x <- settings]
-        mns = map (scopeCreate . fst &&& id) ms
+        mns = map (\x -> (scopeCreate (hseModule x), x)) ms
         hints = (if length ms <= 1 then noModules else id) hints_
         noModules h = h{hintModules = \_ _ -> []} `mappend` mempty{hintModule = \s a b -> hintModules h s [(a,b)]}
 
@@ -70,17 +77,17 @@ removeRequiresExtensionNotes m = \x -> x{ideaNote = filter keep $ ideaNote x}
         keep _ = True
 
 -- | Given a list of settings (a way to classify) and a list of hints, run them over a list of modules.
-executeHints :: [Setting] -> [(Module_, [Comment])] -> [Idea]
+executeHints :: [Setting] -> [ModuleEx] -> [Idea]
 executeHints s = applyHintsReal s (allHints s)
 
 
 -- | Return either an idea (a parse error) or the module. In IO because might call the C pre processor.
-parseModuleApply :: ParseFlags -> [Setting] -> FilePath -> Maybe String -> IO (Either Idea (Module_, [Comment]))
+parseModuleApply :: ParseFlags -> [Setting] -> FilePath -> Maybe String -> IO (Either Idea ModuleEx)
 parseModuleApply flags s file src = do
     res <- parseModuleEx (parseFlagsAddFixities [x | Infix x <- s] flags) file src
     case res of
-        Right m -> return $ Right m
-        Left (ParseError sl msg ctxt) ->
+      Right r -> return $ Right r
+      Left (ParseError sl msg ctxt) ->
             return $ Left $ classify [x | SettingClassify x <- s] $ rawIdeaN Error "Parse error" (mkSrcSpan sl sl) ctxt Nothing []
 
 
