@@ -1,12 +1,15 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE PackageImports #-}
 
 module HSE.All(
     module X,
     CppFlags(..), ParseFlags(..), defaultParseFlags,
     parseFlagsAddFixities, parseFlagsSetLanguage,
-    parseModuleEx, ParseError(..),
-    freeVars, vars, varss, pvars
+    ParseError(..), ModuleEx(..),
+    parseModuleEx, ghcComments,
+    freeVars, vars, varss, pvars,
+    ghcSpanToHSE, ghcSrcLocToHSE
     ) where
 
 import Language.Haskell.Exts.Util hiding (freeVars, Vars(..))
@@ -22,6 +25,7 @@ import Data.List.Extra
 import Data.Maybe
 import Timing
 import Language.Preprocessor.Cpphs
+import Data.Either
 import Data.Set (Set)
 import qualified Data.Map as Map
 import qualified Data.Set as Set
@@ -29,6 +33,38 @@ import System.IO.Extra
 import Data.Functor
 import Prelude
 
+import GHC.Util
+import qualified "ghc-lib-parser" HsSyn
+import qualified "ghc-lib-parser" FastString
+import qualified "ghc-lib-parser" SrcLoc as GHC
+import qualified "ghc-lib-parser" ErrUtils
+import qualified "ghc-lib-parser" Outputable
+import qualified "ghc-lib-parser" Lexer as GHC
+import qualified "ghc-lib-parser" GHC.LanguageExtensions.Type as GHC
+import qualified "ghc-lib-parser" ApiAnnotation as GHC
+
+-- | Convert a GHC source loc into an HSE equivalent.
+ghcSrcLocToHSE :: GHC.SrcLoc -> SrcLoc
+ghcSrcLocToHSE (GHC.RealSrcLoc l) =
+  SrcLoc {
+      srcFilename = FastString.unpackFS (GHC.srcLocFile l)
+    , srcLine = GHC.srcLocLine l
+    , srcColumn = GHC.srcLocCol l
+    }
+ghcSrcLocToHSE (GHC.UnhelpfulLoc _) = noLoc
+
+-- | Convert a GHC source span into an HSE equivalent.
+ghcSpanToHSE :: GHC.SrcSpan -> SrcSpan
+ghcSpanToHSE (GHC.RealSrcSpan s) =
+  SrcSpan {
+      srcSpanFilename = FastString.unpackFS (GHC.srcSpanFile s)
+    , srcSpanStartLine = GHC.srcSpanStartLine s
+    , srcSpanStartColumn = GHC.srcSpanStartCol s
+    , srcSpanEndLine = GHC.srcSpanEndLine s
+    , srcSpanEndColumn = GHC.srcSpanEndCol s
+    }
+ghcSpanToHSE (GHC.UnhelpfulSpan _) = mkSrcSpan noLoc noLoc
+
 vars :: FreeVars a => a -> [String]
 freeVars :: FreeVars a => a -> Set String
 varss, pvars :: AllVars a => a -> [String]
@@ -139,20 +175,111 @@ runCpp (Cpphs o) file x = dropLine <$> runCpphs o file x
         dropLine (line1 -> (a,b)) | "{-# LINE " `isPrefixOf` a = b
         dropLine x = x
 
-
 ---------------------------------------------------------------------
 -- PARSING
 
--- | A parse error from 'parseModuleEx'.
+-- | A parse error.
 data ParseError = ParseError
-    {parseErrorLocation :: SrcLoc -- ^ Location of the error.
-    ,parseErrorMessage :: String -- ^ Message about the cause of the error.
-    ,parseErrorContents :: String -- ^ Snippet of several lines (typically 5) including a @>@ character pointing at the faulty line.
+    { parseErrorLocation :: SrcLoc -- ^ Location of the error.
+    , parseErrorMessage :: String  -- ^ Message about the cause of the error.
+    , parseErrorContents :: String -- ^ Snippet of several lines (typically 5) including a @>@ character pointing at the faulty line.
     }
 
--- | Parse a Haskell module. Applies the C pre processor, and uses best-guess fixity resolution if there are ambiguities.
---   The filename @-@ is treated as @stdin@. Requires some flags (often 'defaultParseFlags'), the filename, and optionally the contents of that file.
-parseModuleEx :: ParseFlags -> FilePath -> Maybe String -> IO (Either ParseError (Module SrcSpanInfo, [Comment]))
+-- | Result of 'parseModuleEx', representing a parsed module.
+data ModuleEx = ModuleEx {
+    hseModule :: Module SrcSpanInfo
+  , hseComments :: [Comment]
+  , ghcModule :: Located (HsSyn.HsModule HsSyn.GhcPs)
+  , ghcAnnotations :: GHC.ApiAnns
+}
+
+-- | Extract a list of all of a parsed module's comments.
+ghcComments :: ModuleEx -> [Located GHC.AnnotationComment]
+ghcComments m = concat (Map.elems $ snd (ghcAnnotations m))
+
+-- | Utility called from 'parseModuleEx' and 'hseFailOpParseModuleEx'.
+mkMode :: ParseFlags -> String -> ParseMode
+mkMode flags file = (hseFlags flags){ parseFilename = file,fixities = Nothing }
+
+-- | Error handler dispatcher. Invoked when HSE parsing has failed.
+failOpParseModuleEx :: String
+                    -> ParseFlags
+                    -> FilePath
+                    -> String
+                    -> SrcLoc
+                    -> String
+                    -> Maybe (GHC.SrcSpan, ErrUtils.MsgDoc)
+                    -> IO (Either ParseError ModuleEx)
+failOpParseModuleEx ppstr flags file str sl msg ghc =
+   case ghc of
+     Just err ->
+       -- GHC error info is available (assumed to have come from a
+       -- 'PFailed'). We prefer to construct a 'ParseError' value
+       -- using that.
+       ghcFailOpParseModuleEx ppstr file str err
+     Nothing ->
+       -- No GHC error info provided. This is the traditional approach
+       -- to handling errors.
+       hseFailOpParseModuleEx ppstr flags file str sl msg
+
+-- | An error handler of last resort. This is invoked when HSE parsing
+-- has failed but apparently GHC has not!
+hseFailOpParseModuleEx :: String
+                       -> ParseFlags
+                       -> FilePath
+                       -> String
+                       -> SrcLoc
+                       -> String
+                       -> IO (Either ParseError ModuleEx)
+hseFailOpParseModuleEx ppstr flags file str sl msg = do
+    flags <- return $ parseFlagsNoLocations flags
+    ppstr2 <- runCpp (cppFlags flags) file str
+    let pe = case parseFileContentsWithMode (mkMode flags file) ppstr2 of
+               ParseFailed sl2 _ -> context (srcLine sl2) ppstr2
+               _ -> context (srcLine sl) ppstr
+    return $ Left $ ParseError sl msg pe
+
+-- | The error handler invoked when GHC parsing has failed.
+ghcFailOpParseModuleEx :: String
+                       -> FilePath
+                       -> String
+                       -> (GHC.SrcSpan, ErrUtils.MsgDoc)
+                       -> IO (Either ParseError ModuleEx)
+ghcFailOpParseModuleEx ppstr file str (loc, err) = do
+   let sl =
+         case loc of
+           GHC.RealSrcSpan r ->
+             SrcLoc { srcFilename = FastString.unpackFS (GHC.srcSpanFile r)
+                     , srcLine = GHC.srcSpanStartLine r
+                     , srcColumn = GHC.srcSpanStartCol r }
+           GHC.UnhelpfulSpan _ ->
+             SrcLoc { srcFilename = file
+                     , srcLine = 1 :: Int
+                     , srcColumn = 1 :: Int }
+       pe = context (srcLine sl) ppstr
+       msg = Outputable.showSDoc baseDynFlags $
+               ErrUtils.pprLocErrMsg (ErrUtils.mkPlainErrMsg baseDynFlags loc err)
+   return $ Left $ ParseError sl msg pe
+
+-- | Produce a pair of lists from a 'ParseFlags' value representing
+-- language extensions to explicitly enable/disable.
+ghcExtensionsFromParseFlags :: ParseFlags
+                             -> ([GHC.Extension], [GHC.Extension])
+ghcExtensionsFromParseFlags ParseFlags {hseFlags=ParseMode {extensions=exts}}=
+   partitionEithers $ mapMaybe toEither exts
+   where
+     toEither ke = case ke of
+       EnableExtension e  -> Left  <$> readExtension (show e)
+       DisableExtension e -> Right <$> readExtension (show e)
+       UnknownExtension ('N':'o':e) -> Right <$> readExtension e
+       UnknownExtension e -> Left <$> readExtension e
+
+-- | Parse a Haskell module. Applies the C pre processor, and uses
+-- best-guess fixity resolution if there are ambiguities.  The
+-- filename @-@ is treated as @stdin@. Requires some flags (often
+-- 'defaultParseFlags'), the filename, and optionally the contents of
+-- that file. This version uses both hs-src-exts AND ghc-lib.
+parseModuleEx :: ParseFlags -> FilePath -> Maybe String -> IO (Either ParseError ModuleEx)
 parseModuleEx flags file str = timedIO "Parse" file $ do
         str <- case str of
             Just x -> return x
@@ -160,24 +287,37 @@ parseModuleEx flags file str = timedIO "Parse" file $ do
                     | otherwise -> readFileUTF8' file
         str <- return $ fromMaybe str $ stripPrefix "\65279" str -- remove the BOM if it exists, see #130
         ppstr <- runCpp (cppFlags flags) file str
-        case parseFileContentsWithComments (mode flags) ppstr of
-            ParseOk (x, cs) -> return $ Right (applyFixity fixity x, cs)
-            ParseFailed sl msg -> do
-                -- figure out the best line number to grab context from, by reparsing
-                -- but not generating {-# LINE #-} pragmas
-                flags <- return $ parseFlagsNoLocations flags
-                ppstr2 <- runCpp (cppFlags flags) file str
-                let pe = case parseFileContentsWithMode (mode flags) ppstr2 of
-                        ParseFailed sl2 _ -> context (srcLine sl2) ppstr2
-                        _ -> context (srcLine sl) ppstr
-                return $ Left $ ParseError sl msg pe
-    where
-        fixity = fromMaybe [] $ fixities $ hseFlags flags
-        mode flags = (hseFlags flags)
-            {parseFilename = file
-            ,fixities = Nothing
-            }
+        let enableDisableExts = ghcExtensionsFromParseFlags flags
+        dynFlags <- parsePragmasIntoDynFlags baseDynFlags enableDisableExts file ppstr
+        case dynFlags of
+          Right ghcFlags ->
+            case (parseFileContentsWithComments (mkMode flags file) ppstr, parseFileGhcLib file ppstr ghcFlags) of
+                (ParseOk (x, cs), POk pst a) ->
+                    let anns =
+                          ( Map.fromListWith (++) $ GHC.annotations pst
+                          , Map.fromList ((GHC.noSrcSpan, GHC.comment_q pst) : GHC.annotations_comments pst)
+                          ) in
+                    return $ Right (ModuleEx (applyFixity fixity x) cs a anns)
+                -- Parse error if GHC parsing fails (see
+                -- https://github.com/ndmitchell/hlint/issues/645).
+                (ParseOk _, PFailed _ loc err) ->
+                    ghcFailOpParseModuleEx ppstr file str (loc, err)
+                (ParseFailed sl msg, pfailed) ->
+                    failOpParseModuleEx ppstr flags file str sl msg $ fromPFailed pfailed
+          Left msg -> do
+            -- Parsing GHC flags from dynamic pragmas in the source
+            -- has failed. When this happens, it's reported by
+            -- exception. It's impossible or at least fiddly getting a
+            -- location so we skip that for now. Synthesize a parse
+            -- error.
+            let loc = SrcLoc file (1 :: Int) (1 :: Int)
+            return $ Left (ParseError loc msg (context (srcLine loc) ppstr))
 
+    where
+        fromPFailed (PFailed _ loc err) = Just (loc, err)
+        fromPFailed _ = Nothing
+
+        fixity = fromMaybe [] $ fixities $ hseFlags flags
 
 -- | Given a line number, and some source code, put bird ticks around the appropriate bit.
 context :: Int -> String -> String
