@@ -1,6 +1,6 @@
 
 module Hint.All(
-    Hint(..), HintBuiltin(..), DeclHint, ModuHint,
+    Hint(..), DeclHint, ModuHint,
     resolveHints, hintRules, builtinHints
     ) where
 
@@ -49,24 +49,24 @@ builtin x = case x of
     HintMonad      -> decl monadHint
     HintLambda     -> decl lambdaHint
     HintBracket    -> decl bracketHint
-    HintNaming     -> decl namingHint
+    HintNaming     -> decl' namingHint
     HintPattern    -> decl patternHint
     HintImport     -> modu importHint
     HintExport     -> modu exportHint
+    HintComment    -> modu commentHint
     HintPragma     -> modu pragmaHint
     HintExtensions -> modu extensionsHint
     HintUnsafe     -> decl unsafeHint
     HintDuplicate  -> mods duplicateHint
-    HintComment    -> comm commentHint
-    HintNewType    -> decl newtypeHint
+    HintNewType    -> decl' newtypeHint
     HintRestrict   -> mempty{hintModule=restrictHint}
     HintSmell      -> mempty{hintDecl=smellHint,hintModule=smellModuleHint}
     where
         wrap = timed "Hint" (drop 4 $ show x) . forceList
         decl f = mempty{hintDecl=const $ \a b c -> wrap $ f a b c}
+        decl' f = mempty{hintDecl'=const $ \a b c -> wrap $ f a b c}
         modu f = mempty{hintModule=const $ \a b -> wrap $ f a b}
         mods f = mempty{hintModules=const $ \a -> wrap $ f a}
-        comm f = mempty{hintComment=const $ \a -> wrap $ f a}
 
 -- | A list of builtin hints, currently including entries such as @\"List\"@ and @\"Bracket\"@.
 builtinHints :: [(String, Hint)]
