@@ -22,6 +22,15 @@ no = \(x -> y) -> z
 yes = (`foo` (bar baz)) -- @Suggestion (`foo` bar baz)
 yes = f ((x)) -- @Warning x
 main = do f; (print x) -- @Suggestion do f print x
+yes = f (x) y -- @Warning x
+no = f (+x) y
+no = f ($x) y
+no = ($x)
+yes = (($x))
+no = ($1)
+yes = (($1)) -- @Warning ($1)
+no = (+5)
+yes = ((+5)) -- @Warning (+5)
 
 -- type bracket reduction
 foo :: (Int -> Int) -> Int
@@ -95,6 +104,7 @@ bracketHint _ _ x =
             x -> x
 
 isPartialAtom :: Exp_ -> Bool
+isPartialAtom (SpliceExp _ IdSplice{}) = True -- might be $x, which was really $ x, but TH enabled misparsed it
 isPartialAtom x = isRecConstr x || isRecUpdate x
 
 -- Dirty, should add to Brackets type class I think
@@ -122,7 +132,7 @@ bracket isPartialAtom root = f Nothing
         -- f (Maybe (index, parent, gen)) child
         f :: (Data (a S), ExactP a, Pretty (a S), Brackets (a S)) => Maybe (Int,a S,a S -> a S) -> a S -> [Idea]
         f Just{} o@(remParens -> Just x) | isAtom x, not $ isPartialAtom x = bracketError msg o x : g x
-        f Nothing o@(remParens -> Just x) | root || isAtom x = (if isAtom x then bracketError else bracketWarning) msg o x : g x
+        f Nothing o@(remParens -> Just x) | root || isAtom x, not $ isPartialAtom x = (if isAtom x then bracketError else bracketWarning) msg o x : g x
         f (Just (i,o,gen)) v@(remParens -> Just x) | not $ needBracket i o x, not $ isPartialAtom x =
           suggest msg o (gen x) [r] : g x
           where
