@@ -23,6 +23,14 @@ instance C [Char]
 foo = [a b] ++ xs -- a b : xs
 foo = [myexpr | True, a] -- [myexpr | a]
 foo = [myexpr | False] -- []
+foo = map f [x + 1 | x <- [1..10]] -- [f (x + 1) | x <- [1..10]]
+foo = [x + 1 | x <- [1..10], feature] -- [x + 1 | feature, x <- [1..10]]
+foo = [x + 1 | x <- [1..10], even x]
+foo = [x + 1 | x <- [1..10], even x, dont_reoder_guards]
+foo = [x + 1 | x <- [1..10], let y = even x, y]
+foo = [x + 1 | x <- [1..10], let q = even 1, q] -- [x + 1 | let q = even 1, q, x <- [1..10]]
+foo = [fooValue | Foo{..} <- y, fooField]
+issue619 = [pkgJobs | Pkg{pkgGpd, pkgJobs} <- pkgs, not $ null $ C.condTestSuites pkgGpd]
 </TEST>
 -}
 
@@ -30,9 +38,11 @@ module Hint.List(listHint) where
 
 import Control.Applicative
 import Hint.Type
+import Data.List.Extra
 import Data.Maybe
 import Prelude
-import Refact.Types
+import Refact.Types hiding (SrcSpan)
+import qualified Refact.Types as R
 
 
 listHint :: DeclHint
@@ -47,15 +57,39 @@ listDecl x =
 
 listComp :: Exp_ -> [Idea]
 listComp o@(ListComp a e xs)
-    | "False" `elem` cons = [suggest "Short-circuited list comprehension" o (List an []) []]
-    | "True" `elem` cons = [suggest "Redundant True guards" o o2 []]
+    | "False" `elem` cons = [suggest "Short-circuited list comprehension" o o' (suggestExpr o o')]
+    | "True" `elem` cons = [suggest "Redundant True guards" o o2 (suggestExpr o o2)]
+    | xs /= ys = [suggest "Move guards forward" o o3 (suggestExpr o o3)]
     where
+        ys = moveGuardsForward xs
+        o' = List an []
         o2 = ListComp a e $ filter ((/= Just "True") . qualCon) xs
+        o3 = ListComp a e ys
         cons = mapMaybe qualCon xs
         qualCon (QualStmt _ (Qualifier _ (Con _ x))) = Just $ fromNamed x
         qualCon _ = Nothing
+listComp o@(view -> App2 mp f (ListComp a e xs)) | mp ~= "map" =
+    [suggest "Move map inside list comprehension" o o2 (suggestExpr o o2)]
+    where o2 = ListComp a (App an (paren f) (paren e)) xs
 listComp _ = []
 
+suggestExpr :: Exp_ -> Exp_ -> [Refactoring R.SrcSpan]
+suggestExpr o o2 = [Replace Expr (toSS o) [] (prettyPrint o2)]
+
+-- Move all the list comp guards as far forward as they can go
+moveGuardsForward :: [QualStmt S] -> [QualStmt S]
+moveGuardsForward = reverse . f [] . reverse
+    where
+        f guards (x@(QualStmt _ (Generator _ p _)):xs) = reverse stop ++ x : f move xs
+            where (move, stop) = span (if any isPFieldWildcard (universeS x) then const False else \x -> pvars p `disjoint` vars' x) guards
+        f guards (x@(QualStmt _ Qualifier{}):xs) = f (x:guards) xs
+        f guards (x@(QualStmt _ LetStmt{}):xs) = f (x:guards) xs
+        f guards xs = reverse guards ++ xs
+
+        -- the type QualStmt doesn't have a Var instance, so fake something that works
+        vars' x = [prettyPrint a | Var _ a <- universeS x]
+
+
 -- boolean = are you in a ++ chain
 listExp :: Bool -> Exp_ -> [Idea]
 listExp b (fromParen -> x) =
@@ -77,20 +111,20 @@ isAppend _ = False
 
 
 checks = let (*) = (,) in drop 1 -- see #174
-         ["Use string literal" * useString
-         ,"Use list literal" * useList
-         ,"Use :" * useCons
-         ]
+    ["Use string literal" * useString
+    ,"Use list literal" * useList
+    ,"Use :" * useCons
+    ]
 
 pchecks = let (*) = (,) in drop 1 -- see #174
-          ["Use string literal pattern" * usePString
-          ,"Use list literal pattern" * usePList
-          ]
+    ["Use string literal pattern" * usePString
+    ,"Use list literal pattern" * usePList
+    ]
 
 
 usePString (PList _ xs) | xs /= [], Just s <- mapM fromPChar xs =
-  let literal = PLit an (Signless an) $ String an s (show s)
-  in Just (literal, [], prettyPrint literal)
+    let literal = PLit an (Signless an) $ String an s (show s)
+    in Just (literal, [], prettyPrint literal)
 usePString _ = Nothing
 
 usePList =
