@@ -4,7 +4,8 @@ module Util(
     defaultExtensions,
     forceList,
     gzip, universeParentBi,
-    exitMessage, exitMessageImpure
+    exitMessage, exitMessageImpure,
+    getContentsUTF8
     ) where
 
 import Data.List
@@ -36,6 +37,12 @@ exitMessageImpure :: String -> a
 exitMessageImpure = unsafePerformIO . exitMessage
 
 
+getContentsUTF8 :: IO String
+getContentsUTF8 = do
+    hSetEncoding stdin utf8
+    getContents
+
+
 ---------------------------------------------------------------------
 -- DATA.GENERICS
 
